@@ -64,7 +64,7 @@ use YAML::Syck;
 use Test::System::Output::Factory;
 use TAP::Harness;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 =head1 Attributes
 
@@ -127,7 +127,7 @@ has 'nodes' => (
 
 A write access string that has the format of how the tests should be presented,
 please refer to the modules available under L<Test::System::Output> or in your
-custom factory (C<custom_factory> attribute) class if this is the case.
+custom factory (C<format_factory_class> attribute) class if this is the case.
 
 =cut
 has 'format' => (
@@ -145,19 +145,19 @@ A list of available formats, read only.
 has 'available_formats' => (
         is => 'ro',
         isa => => 'HashRef',
-        default => \&_generate_output_types_hash
+        default => \&_fill_available_formats_hash
         );
 
-=item B<custom_factory>
+=item B<format_factory_class>
 
 If you want to use your own Factory for creating your output you can set this
 to your class name (B<NOT the object>).
 
 =cut
-has 'custom_factory' => (
+has 'format_factory_class' => (
         is => 'rw',
         isa => 'Str',
-        trigger => \&_generate_output_types_hash
+        trigger => \&_fill_available_formats_hash
         );
 
 =item B<harness>
@@ -201,11 +201,13 @@ generated by the L<TAP::Parser::Aggregator>.
 
 =over 4
 
-=item * passed: Number of passed tests
+=item * passed: Total of passed tests
 
 =item * total: Total number of tests (passed or not)
 
-=item * skipped: Tests that required a SKIP
+=item * skipped: Total of skipped tests
+
+=item * failed: Total of failed tests.
 
 =back
 
@@ -220,14 +222,26 @@ has 'results' => (
 A string (read-only) that contains a word that describes the status of all
 the tests. This value is also generated by L<TAP::Parser::Aggregator>.
 
-=back
-
 =cut
 has 'status' => (
         is => 'ro',
         isa => 'Str'
         );
 
+=item B<show_warnings>
+
+By default we show warnings of all things that can make your tests to run
+in a different way than expected.
+
+=back
+
+=cut
+has 'show_warnings' => (
+        is => 'rw',
+        isa => 'Bool',
+        default => 1
+        );
+
 =head1 Methods
 
 =head2 B<runtests(@tests,%options)>
@@ -300,14 +314,24 @@ sub runtests {
                 if (defined $self->available_tests->{$_}->{'description'}) {
                     $description = $self->available_tests->{$_}->{'description'};
                 }
+            } else {
+                warn "$_ is not listed in the available tests list, forcing it"
+                    if $self->show_warnings;
             }
             if (!-f $code) {
-                warn "$_ was not found inside the test groups file, skipping";
+                if ($code eq $_) {
+                    warn "$code does not exist as a file, skipping"
+                        if $self->show_warnings;
+                } else {
+                    warn "$_ test file ($code) does not exist as a file, skipping"
+                        if $self->show_warnings;
+                }
                 next;
             }
         } else {
             if (!-f $_) {
-                warn "$_ was not found on a test YAML file or as a file, skipping";
+                warn "$_ test does not exist as a file, skipping"
+                    if $self->show_warnings;
                 next;
             }
         }
@@ -316,8 +340,8 @@ sub runtests {
         }
     }
     my $factory_class = 'Test::System::Output::Factory';
-    if ($self->custom_factory) {
-        $factory_class = $self->custom_factory;
+    if ($self->format_factory_class) {
+        $factory_class = $self->format_actory_class;
     }
     my $formatter_class = "$factory_class"->get_registered_class(
             $self->format
@@ -372,8 +396,8 @@ the C<test_groups>.
 
 =cut
 sub pretests_verification {
-    my ($selv, $tests, $options) = @_;
-    }
+    my ($self, $tests, $options) = @_;
+}
 
 
 =head2 B<what_happened($parser, $result_test)>
@@ -403,15 +427,21 @@ sub what_happened {
     # We are going to do a nasty thing with TAP::Parser, we are going to create
     # a new attribute/item that we can use later to get detailed results in the
     # format we want.
-    if (!defined $parser->{'test_system_results'}) {
-        $parser->{'test_system_results'} = [];
+    if (!defined $parser->{'test_system_details'}) {
+        $parser->{'test_system_details'} = [];
+        $parser->{'test_system_notes'} = [];
     }
 
     if ($result_test->{'type'} eq 'test') {
-        push(@{$parser->{'test_system_results'}}, {
+        push(@{$parser->{'test_system_details'}}, {
                 num         => $result_test->{'test_num'},
                 description => $result_test->{'description'},
-                raw         => $result_test->{'raw'}
+                raw         => $result_test->{'raw'},
+                });
+    } elsif ($result_test->{'type'} eq 'comment') { # for notes()
+        push(@{$parser->{'test_system_notes'}}, {
+                comment  => $result_test->{'comment'},
+                raw      => $result_test->{'raw'},
                 });
     }
 }
@@ -431,7 +461,7 @@ sub set_results {
     my ($self, $parser_aggregator) = @_;
 
     $self->{'status'} = $parser_aggregator->get_status;
-    foreach my $key (qw(passed total skipped)) {
+    foreach my $key (qw(passed total skipped failed)) {
         $self->{'results'}->{$key} = $parser_aggregator->{$key};
     }
 
@@ -450,15 +480,19 @@ sub set_results {
     }
 
     $self->{'results'}->{'details'} = {};
+    $self->{'results'}->{'notes'} = {};
     foreach my $key (keys %{$parser_aggregator->{'parser_for'}}) {
         my $parser = $parser_aggregator->{'parser_for'}->{$key};
-        if (defined $parser->{'test_system_results'}) {
-            my $id_to_store = $key;
-            if (defined $tests_ids{$id_to_store}) {
-                $id_to_store = $tests_ids{$id_to_store};
+        foreach my $want (qw(details notes)) {
+            my $parser_info_key = 'test_system_' . $want;
+            if (defined $parser->{$parser_info_key}) {
+                my $id_to_store = $key;
+                if (defined $tests_ids{$id_to_store}) {
+                    $id_to_store = $tests_ids{$id_to_store};
+                }
+                $self->{'results'}->{$want}->{$id_to_store} =
+                    $parser->{$parser_info_key};
             }
-            $self->{'results'}->{'details'}->{$id_to_store} =
-                $parser->{'test_system_results'};
         }
     }
 }
@@ -549,14 +583,15 @@ sub get_tests_from_test_plan {
     my ($self, $yaml_file, $do_not_fill_parameters, $do_not_fill_nodes) = @_;
 
     if (!-f $yaml_file) {
-        warn "YAML file ($yaml_file) does not exist, skipping the read";
+        confess "YAML file ($yaml_file) does not exist";
         return;
     }
 
     my @tests;
     my $data = LoadFile($yaml_file);
     if (!defined $data->{'tests'}) {
-        warn "YAML file ($yaml_file) does not have any tests to execute";
+        warn "YAML file ($yaml_file) does not have any tests to execute, skipping"
+            if $self->show_warnings;
         return;
     } else {
         @tests = @{$data->{'tests'}};
@@ -583,10 +618,11 @@ sub _read_test_groups_file {
     my ($self, $yaml_file) = @_;
     
     if (!-f $yaml_file) {
-        confess "The YAML file ($yaml_file) does not exist";
+        confess "The YAML file ($yaml_file) does not exist"
+            if $self->show_warnings;
     }
 
-    $self->{tests_yaml} = $yaml_file;
+    $self->{test_groups} = $yaml_file;
     my $testdir = dirname($yaml_file);
 
     my $tests = LoadFile($yaml_file);
@@ -626,15 +662,15 @@ sub _verify_format {
     $self->{'format'} = $format;
 }
 
-# Generates the output type hash reference by checking the registered factory
-# types of Test::System::Output::Factory or in the custom Factory class
-sub _generate_output_types_hash {
+# Fills the available_formats hash with the classes registered in the
+# factory
+sub _fill_available_formats_hash {
     my ($self, $new_factory) = @_;
 
     if (!$new_factory) {
         $new_factory = 'Test::System::Output::Factory';
     } else {
-        $self->{custom_factory} = $new_factory;
+        $self->{format_factory_class} = $new_factory;
     }
     my @registered_types = "$new_factory"->get_registered_types;
 
@@ -663,4 +699,3 @@ it under the same terms as Perl itself.
 =cut
 
 1;
-
